@@ -5,16 +5,17 @@ use strict;
 use warnings;
 use feature 'state';
 
-use English               qw( -no_match_vars );
-use Exporter 5.57         qw( import );
-use File::DataClass::Constants;
-use Hash::Merge           qw( merge );
-use List::Util            qw( first );
+use English                    qw( -no_match_vars );
+use Exporter 5.57              qw( import );
+use File::DataClass::Constants qw( CYGWIN EVIL EXCEPTION_CLASS STORAGE_BASE
+                                   STORAGE_EXCEPTIONS );
+use Hash::Merge                qw( merge );
+use List::Util                 qw( first );
 use Module::Pluggable::Object;
-use Module::Runtime       qw( require_module );
-use Scalar::Util          qw( blessed );
+use Module::Runtime            qw( require_module );
+use Scalar::Util               qw( blessed );
 use Try::Tiny;
-use Unexpected::Functions qw( is_class_loaded );
+use Unexpected::Functions      qw( is_class_loaded );
 
 our @EXPORT_OK    = qw( ensure_class_loaded extension_map first_char
                         is_arrayref is_coderef is_hashref is_member
